@@ -4,13 +4,13 @@
 #' @param x an object with simualted residuals created by \code{\link{simulateResiduals}}
 #' @param rank if T (default), the values of pred will be rank transformed. This will usually make patterns easier to spot visually, especially if the distribution of the predictor is skewed. 
 #' @param ... further options for \code{\link{plotResiduals}}. Consider in particular parameters quantreg, rank and asFactor. xlab, ylab and main cannot be changed when using plotSimulatedResiduals, but can be changed when using plotResiduals.
-#' @details The function creates two plots. To the left, a qq-uniform plot to detect deviations from overall uniformity of the residuals (calling \code{\link{plotQQunif}}), and to the right, a plot of residuals against predicted values (calling \code{\link{plotResiduals}}). For a correctly specified model, we would expect 
+#' @details The function creates two plots. To the left, a qq-uniform plot to detect deviations from overall uniformity of the residuals (calling \code{\link{plotQQunif}}), and to the right, a plot of residuals against predicted values (calling \code{\link{plotResiduals}}). Outliers are highlighted in red (for more on oultiers, see \code{\link{testOutliers}}). For a correctly specified model, we would expect 
 #' 
 #' a) a straight 1-1 line in the uniform qq-plot -> evidence for an overal uniform (flat) distribution of the residuals
 #' 
 #' b) uniformity of residuals in the vertical direction in the res against predictor plot
 #' 
-#' Deviations of this can be interpreted as for a liner regression. See the vignette for detailed examples. 
+#' Deviations of this can be interpreted as for a linear regression. See the vignette for detailed examples. 
 #' 
 #' To provide a visual aid in detecting deviations from uniformity in y-direction, the plot of the residuals against the predited values also performs an (optional) quantile regression, which provides 0.25, 0.5 and 0.75 quantile lines across the plots. These lines should be straight, horizontal, and at y-values of 0.25, 0.5 and 0.75. Note, however, that some deviations from this are to be expected by chance, even for a perfect model, especially if the sample size is small. See further comments on this plot, and options, in \code{\link{plotResiduals}}
 #' 
@@ -27,9 +27,11 @@ plot.DHARMa <- function(x, rank = TRUE, ...){
   
   plotQQunif(x)
   
-  xla = ifelse(rank, "Predicted values (rank transformed)", "Predicted values")
+  xlab = checkDots("xlab", ifelse(rank, "Predicted values (rank transformed)", "Predicted values"), ...)
+  ylab = checkDots("ylab", "Standardized residual", ...)
+  main = checkDots("main", "Residual vs. predicted\n lines should match", ...)
   
-  plotResiduals(pred = x, residuals = NULL, xlab = xla, ylab = "Standardized residual", main = "Residual vs. predicted\n lines should match", rank = rank, ...)
+  plotResiduals(pred = x, residuals = NULL, xlab = xlab, ylab = ylab, main = main, rank = rank, ...)
   
   mtext("DHARMa scaled residual plots", outer = T)
   
@@ -37,6 +39,30 @@ plot.DHARMa <- function(x, rank = TRUE, ...){
 }
 
 
+#' Histogram of DHARMa residuals
+#' 
+#' The function produces a histogram from a DHARMa output
+#' 
+#' @param x a DHARMa simulation output (class DHARMa)
+#' @param ... arguments to be passed on to hist. Breaks and col are fixed. 
+#' @seealso \code{\link{plotSimulatedResiduals}}, \code{\link{plotResiduals}}
+#' @example inst/examples/plotsHelp.R
+#' @export
+hist.DHARMa <- function(x, ...){
+  
+  val = x$scaledResiduals
+  val[val == 0] = -0.01
+  val[val == 1] = 1.01
+  
+  breaks = checkDots("breaks", seq(-0.02, 1.02, len = 53), ...)
+  col = checkDots("col", c("red",rep("lightgrey",50), "red"), ...)
+  main = checkDots("main", "Hist of DHARM residuals\nOutliers are marked red", ...)   
+
+  hist(val, breaks = breaks, col = col, main = main, ...)
+}
+
+
+
 
 #' DHARMa standard residual plots
 #' 
@@ -60,20 +86,28 @@ plotSimulatedResiduals <- function(simulationOutput, ...){
 #' 
 #' @param simulationOutput a DHARMa simulation output (class DHARMa)
 #' @param testUniformity if T, the function \code{\link{testUniformity}} will be called and the result will be added to the plot
+#' @param testOutliers if T, the function \code{\link{testOutliers}} will be called and the result will be added to the plot
+#' @param ... arguments to be passed on to \code{\link[gap]{qqunif}}
 #' 
 #' @details the function calls qqunif from the R package gap to create a quantile-quantile plot for a uniform distribution.  
 #' @seealso \code{\link{plotSimulatedResiduals}}, \code{\link{plotResiduals}}
 #' @example inst/examples/plotsHelp.R
 #' @export
-plotQQunif <- function(simulationOutput, testUniformity = T){
+plotQQunif <- function(simulationOutput, testUniformity = T, testOutliers = T, ...){
   
   if(class(simulationOutput) != "DHARMa") stop("DHARMa::plotQQunif wrong argument, simulationOutput must be a DHARMa object!")
 
-  gap::qqunif(simulationOutput$scaledResiduals,pch=2,bty="n", logscale = F, col = "black", cex = 0.6, main = "QQ plot residuals", cex.main = 1)
+  gap::qqunif(simulationOutput$scaledResiduals,pch=2,bty="n", logscale = F, col = "black", cex = 0.6, main = "QQ plot residuals", cex.main = 1, ...)
+  
   if(testUniformity == TRUE){
     temp = testUniformity(simulationOutput, plot = F)
     legend("topleft", c(paste("KS test: p=", round(temp$p.value, digits = 5)), paste("Deviation ", ifelse(temp$p.value < 0.05, "significant", "n.s."))), text.col = ifelse(temp$p.value < 0.05, "red", "black" ), bty="n")     
   }
+  
+  if(testOutliers == TRUE){
+    temp = testOutliers(simulationOutput, plot = F)
+    legend("bottomright", c(paste("Outlier test: p=", round(temp$p.value, digits = 5)), paste("Deviation ", ifelse(temp$p.value < 0.05, "significant", "n.s."))), text.col = ifelse(temp$p.value < 0.05, "red", "black" ), bty="n")     
+  }
 }
 
 
@@ -85,9 +119,9 @@ plotQQunif <- function(simulationOutput, testUniformity = T){
 #' @param residuals residuals values. Leave empty if pred is a DHARMa object
 #' @param quantreg whether to perform a quantile regression on 0.25, 0.5, 0.75 on the residuals. If F, a spline will be created instead. Default NULL chooses T for nObs < 2000, and F otherwise. 
 #' @param rank if T, the values of pred will be rank transformed. This will usually make patterns easier to spot visually, especially if the distribution of the predictor is skewed. If pred is a factor, this has no effect. 
-#' @param asFactor should the predictor variable converted into a factor
-#' @param ... additional arguments to plot
-#' @details For a correctly specified model, we would expect uniformity in y direction when plotting against any predictor.
+#' @param asFactor should the predictor variable be treated as a factor. Default is to choose this for <10 unique predictions, as long as enough predictions are available to draw a boxplot.
+#' @param ... additional arguments to plot / boxplot. 
+#' @details Plots residuals against a predictor. Outliers are highlighted in red (for more on oultiers, see \code{\link{testOutliers}}). For a correctly specified model, we would expect uniformity in y direction when plotting against any predictor.
 #' 
 #' To provide a visual aid in detecting deviations from uniformity in y-direction, the plot of the residuals against the predited values also performs an (optional) quantile regression, which provides 0.25, 0.5 and 0.75 quantile lines across the plots. These lines should be straight, horizontal, and at y-values of 0.25, 0.5 and 0.75. Note, however, that some deviations from this are to be expected by chance, even for a perfect model, especially if the sample size is small.
 #' 
@@ -98,7 +132,7 @@ plotQQunif <- function(simulationOutput, testUniformity = T){
 #' @seealso \code{\link{plotSimulatedResiduals}}, \code{\link{plotQQunif}}
 #' @example inst/examples/plotsHelp.R
 #' @export
-plotResiduals <- function(pred, residuals = NULL, quantreg = NULL, rank = FALSE, asFactor = FALSE, ...){
+plotResiduals <- function(pred, residuals = NULL, quantreg = NULL, rank = F, asFactor = NULL, ...){
   
   # conversions from DHARMa 
   if(class(pred) == "DHARMa"){
@@ -118,28 +152,31 @@ plotResiduals <- function(pred, residuals = NULL, quantreg = NULL, rank = FALSE,
       stop("DHARMa::plotResiduals - residuals and predictor do not have the same length. ")        
     }
   }
-
-  if (asFactor) pred = as.factor(pred)
   
   if(!is.factor(pred)){
-    nuniq = length(unique(pred))
-    ndata = length(pred)
-    if(nuniq < 10 & ndata / nuniq > 10) message("DHARMa::plotResiduals - low number of unique predictor values, consider setting asFactor = T")
-    if(nuniq < 10 & ndata / nuniq > 10) message("DHARMa::plotResiduals - low number of unique predictor values, consider setting asFactor = T")
-    # this rank tranforms the predictor
+
     if (rank == T){
       pred = rank(pred, ties.method = "average")
       pred = pred / max(pred)          
     } 
-  } else {
-    # if (rank == T) warning("DHARMa::plotResiduals - predictor is a factor, rank = T has no effect")
+        
+    nuniq = length(unique(pred))
+    ndata = length(pred)  
+    if(is.null(asFactor)) asFactor = (nuniq == 1) | (nuniq < 10 & ndata / nuniq > 10)
+    if (asFactor) pred = factor(pred)
   }
   
-
-  
   if(is.null(quantreg)) if (length(res) > 2000) quantreg = FALSE else quantreg = TRUE
+
+  defaultCol = ifelse(res == 0 | res == 1, 2,1)   
+  defaultPch = ifelse(res == 0 | res == 1, 8,1)   
+
+  col = checkDots("col", defaultCol, ...)
+  pch = checkDots("pch", defaultPch, ...)
+  
+  if(is.factor(pred)) plot(res ~ pred, ylim = c(0,1), axes = FALSE, ...)
+  else plot(res ~ pred, ylim = c(0,1), axes = FALSE, col = col, pch = pch, ...)
   
-  plot(res ~ pred, ylim = c(0,1), axes = FALSE, ...)
   axis(1)
   axis(2, at=c(0, 0.25, 0.5, 0.75, 1))
   abline(h = c(0.25, 0.5, 0.75), col = "black", lwd = 0.5, lty = 2)
@@ -198,6 +235,7 @@ plotConventionalResiduals <- function(fittedModel){
 
 
 
+
 # 
 # 
 # if(quantreg == F){
