@@ -1,5 +1,5 @@
 testData = createData(sampleSize = 200, family = poisson(), 
-                      randomEffectVariance = 0, numGroups = 5)
+                      randomEffectVariance = 1, numGroups = 5)
 fittedModel <- glm(observedResponse ~ Environment1, 
                    family = "poisson", data = testData)
 simulationOutput <- simulateResiduals(fittedModel = fittedModel)
@@ -11,10 +11,12 @@ simulationOutput <- simulateResiduals(fittedModel = fittedModel)
 
 plot(simulationOutput, quantreg = FALSE)
 
-#############  qq plot  ######################
+#############  Distribution  ######################
 
 plotQQunif(simulationOutput = simulationOutput)
 
+hist(simulationOutput )
+
 #############  residual plots  ###############
 
 # rank transformation, using a simulationOutput
